// this :
#include "Types.h"

// Slash:
#include <Slash/Data/IIterator.h>
#include <Slash/Tools/F77.h>

// Lib :
#include <Lib/Value.h>

// Inventor :
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoLightModel.h>

// HEPVis :
#include <HEPVis/SbPolyhedron.h>
#include <HEPVis/SbStyle.h>
#include <HEPVis/misc/SoStyleCache.h>
#include <HEPVis/nodes/SoSceneGraph.h>
#include <HEPVis/nodes/SoPolyhedron.h>
#include <HEPVis/nodes/SoHighlightMaterial.h>
#include <HEPVis/nodekits/SoRegion.h>

// G3Lab :
#include <G3Lab/gmanip.h>
#include <G3Lab/Iterator.h>

// G3Simple :
#ifdef WIN32
#define simini SIMINI
#define simevt SIMEVT
#define simtri SIMTRI
#define ibkget IBKGET
#else
#define simini simini_
#define simevt simevt_
#define simtri simtri_
#define ibkget ibkget_
#endif

extern "C" {
  void Slash_stdcall simini();
  void Slash_stdcall simevt(int*);
  void Slash_stdcall simtri();
  int Slash_stdcall ibkget(int*,int*);
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

G3Simple::EventType::EventType(Slash::Core::ISession& aSession)
:Lib::BaseAccessor(aSession.printer())
,fSession(aSession)
,fType("Event")
{
  addProperty("run",Lib::Property::INTEGER,16);
  addProperty("event",Lib::Property::INTEGER,16);
}

G3Simple::EventType::~EventType() {}

std::string G3Simple::EventType::name() const {return fType;}

namespace G3Simple {
  // One shoot only iterator !
  class Event_Iterator : public virtual Slash::Data::IIterator {
  public: //Lib::IIterator
    virtual Slash::Data::IAccessor::Data object() { 
      return (Slash::Data::IAccessor::Data)fEvent; 
    }
    virtual void next() {fEvent = 0;}
    virtual void* tag() {return 0;}
  public:
    Event_Iterator():fEvent(1){}
    virtual ~Event_Iterator(){}
  private:
    int fEvent;
  };
}

Slash::Data::IIterator* G3Simple::EventType::iterator() {
  return new G3Simple::Event_Iterator();
}

Slash::Core::IValue* G3Simple::EventType::findValue(Slash::Data::IAccessor::Data,const std::string& aName,void*) {
  if(aName=="event") {
    int iobj = 1;
    int iprop = 1;
    int event  = ibkget(&iobj,&iprop);
    return new Lib::Value(event);
  } else if(aName=="run") {
    int iobj = 1;
    int iprop = 2;
    int run = ibkget(&iobj,&iprop);
    return new Lib::Value(run);
  } else {
    return new Lib::Value();
  }
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

G3Simple::TOWE_Type::TOWE_Type(Slash::Core::ISession& aSession)
:OnX::InventorAccessor(aSession)
,fSession(aSession)
,fType("TOWE")
{
  addProperty("layer",Lib::Property::INTEGER);
  addProperty("slice",Lib::Property::INTEGER);
  addProperty("tower",Lib::Property::INTEGER);
  addProperty("energy",Lib::Property::DOUBLE);
}

G3Simple::TOWE_Type::~TOWE_Type() {G3Lab::clear_ids(f_ids);}

void* G3Simple::TOWE_Type::cast(const std::string& aClass) const {
  if_Lib_SCast(G3Simple::TOWE_Type)
  else return OnX::InventorAccessor::cast(aClass);
}

std::string G3Simple::TOWE_Type::name() const {return fType;}

void G3Simple::TOWE_Type::fill_ids() {
  int objn = G3Lab::gmanip::detectorNumberOfHits("ECAL","TOWE");
  G3Lab::fill_ids(objn,f_ids);
}

Slash::Data::IIterator* G3Simple::TOWE_Type::iterator() {
  return new G3Lab::Iterator(f_ids);
}

Slash::Core::IValue* G3Simple::TOWE_Type::findValue(Slash::Data::IAccessor::Data aIdentifier,const std::string& aName,void*) {
  int* obj = (int*)aIdentifier;
  std::vector<int> vols;
  std::vector<float> hits;
  G3Lab::gmanip::detectorIthHit("ECAL","TOWE",*obj,vols,hits);
  if(aName=="layer") {
    return new Lib::Value(vols[0]);
  } else if(aName=="slice") {
    return new Lib::Value(vols[1]);
  } else if(aName=="tower") {
    return new Lib::Value(vols[2]);
  } else if(aName=="energy") {
    return new Lib::Value((double)hits[0]);
  } else {
    return new Lib::Value();
  }
}

void G3Simple::TOWE_Type::visualize(Slash::Data::IAccessor::Data aIdentifier,void*) {
  if(!fSoRegion) return;

  int* obj = (int*)aIdentifier;

  std::vector<int> vols;
  std::vector<float> hits;
  if(!G3Lab::gmanip::detectorIthHit("ECAL","TOWE",*obj,vols,hits)) return;

  // Build name (for picking) :
  char s[128];
  ::sprintf(s,"TOWE/0x%lx",obj);

  SoSceneGraph* separator = new SoSceneGraph;
  separator->setString(s);
  
  separator->addChild(fSoGC.getHighlightMaterial());

  int layer = vols[0];
  int slice = vols[1];
  int tower = vols[2];

  double z = -200 + 10 * (slice-1);    // z:   40 slices [-200,200]
  double phi = (tower-1) * 10;         // phi: 36 towers [0,360]
  double rad = 100 + (layer-1) * 20;   // rad:  3 layers [100,160]
          
  //printf ("debug:%d:%d %d %d:%g %g %g\n",
    //voln,vols[0],vols[1],vols[2],z,phi,r);
  //printf ("debug:%d %g:%d %g\n",hitn,hits[0],Class.rep,Class.norm);

  double rmin = rad;
  double rmax = rad+20;
  double dz = 5;
  double phmin = phi;
  phmin *= M_PI/180.;
  double phmax = phi+10;
  phmax *= M_PI/180.;
  SbPolyhedronTubs result(rmin,rmax,dz,phmin,phmax-phmin);

  // Rotated then placed :
  SbMatrix matrix;
  matrix.makeIdentity();
  result.Transform(HEPVis::SbRotation(SbVec3d(1, 0, 0),0),SbVec3d(0,0,z+5));

  separator->addChild(fSoGC.getLightModel());

  SoPolyhedron* polyhedron = new SoPolyhedron(result);
  const SbString& modeling = fSoGC.getModeling();
  if(modeling==SbModeling_wire_frame)
    polyhedron->solid.setValue(FALSE);
  else
    polyhedron->solid.setValue(TRUE);
  separator->addChild(polyhedron);

/*herisson
  double energy = hits[0];
  double angle = phi * M_PI/180.;
  double bx = r * ::cos(angle);
  double by = r * ::sin(angle);
  double bz = z + 5;
  double b = 1. / ::sqrt(bx * bx + by * by);
  double f = 1. + energy * b * Class.norm;
  GoAddLineToNode(gobject,bx,by,bz,bx * f, by * f,bz);
  */
 
  // Send scene graph to the viewing region :
  fSoRegion->doIt(SbAddNode(separator,"dynamicScene"));
}
