/* ----------------------------HEPVis----------------------------------------*/
/*                                                                           */
/* Class:            Twig                                                    */
/*                                                                           */
/* Description:      OIV / Motif tree structure to control visibility, etc.  */
/*                                                                           */
/* Authors:          G. Alverson, L. Taylor, T.Paul       20 Oct 1998        */
/*                                                                           */
/*---------------------------------------------------------------------------*/


#ifndef INCLUDE_Twig_
#define INCLUDE_Twig_

#include <Inventor/SbString.h>
#include <Inventor/nodes/SoSelection.h>

#include <X11/IntrinsicP.h>

#include <iostream>
#include <vector>
                                                                
class Twig;               //forward reference for the list
class TwigCallbackStruct; //for use with X callbacks

typedef std::vector<Twig *> List_of_Twigs;

//! The standard separator between Twigs is the SoSeparator
typedef SoSeparator TwigSeparator;

// Twig
/*! \class Twig Twig.h HEPVis/Twig.h
 *  \brief A class to provide a link between a user-specified hierarchy
 *   and a set of control widgets.
 *
 *  \author G. Alverson, T. Paul, L. Taylor 
 *  \date 20 October 1998
 */         
class Twig
{
public:

/*! Constructor: 
 *  \arg \c name The name to be displayed for the Twig
 *  \arg \c visibility Default setting of the visibility flag
 *  \arg \c cascade Default setting for the cascade flag
 
 */       
        Twig              (SbString name, Bool visibility=TRUE, Bool cascade=TRUE);

        // Destructor
        // ----------
//! Destructor        
	virtual ~Twig             () { } 

        // Public methods	
        // --------------

//! add a Twig as a child
	void                addChild(Twig*);

//! returns OIV seperator node associated to twig
	TwigSeparator*      getIvNode() const                 {return oiv_node;}        
//! set visibility of twig  
        void                setVisibility (Bool b)            {ion = b;}                
//! get the visibility status of the twig
        Bool                getVisibility () const            {return ion;}   
//! set toggle button cascade 
	void                setCascade  (Bool c)              {icascade = c;}           
//! define the parent widget for this twig 
        void                setParentWidget (Widget w)        {theParentWidget = w;}
//! return the parent widget for this twig  
	Widget              getParentWidget ()  const         {return theParentWidget;}
//! return the name of this twig 
	SbString            getName() const                   {return nameOfTwig;}
//! set the name of this twig
	void                setName(SbString name);            
//! return the number of children connected to this twig
	int                 numberOfChildren() const;
//! get a pointer to the i-th child of this twig
	Twig*               getChildByIndex(int iChild) const;
//! method to perform an ASCII dump of twig structure	
	void                printTree() ;  
//! invoke drawMe method of all subtwigs of this twig                            
	void                drawTree() ;  
//! destroy TwigSeparators (NOT Twigs) + recreate TwigSeparators                             
	void                clearTree();                  
//! destroy Motif widgets for top twig in tree + recreate             
	void                remakeWidgetTree(Twig*);                  
//! destroy the entire twig tree. 
	void                destroyTree();                             
//! create the dialog Widget
        int                 createShowDialog  ();
//! accessor for the current node        
 static TwigSeparator       *getCurrentIvNode ()              {return Twig::theCurrentIvNode;}
//! the callback for the selection operation
 static void                selectionCB  (void *theCBStruct, SoPath *selectionPath);
//! the callback for a pick operation
 static SoPath              *pickCB      (void *theCBStruct, const SoPickedPoint *pick);
//! accessor for the current picked point
 const  SoPickedPoint       *getCurrentPick() const;  
//! sets the current picked point
        void                setCurrentPick(const SoPickedPoint *pick);  
//! extractor for output    
 friend std::ostream& operator << (std::ostream& os, const Twig& t);
//! set the default output stream
        void                setOstream(std::ostream& os);
protected:
	SbString            nameOfTwig;             //!< name of the twig
	Bool                ion;                    //!< display flag
	Bool                icascade;               //!< flag for cascading toggle buttons
	int                 id;                     //!< id of mirror graphics object
        List_of_Twigs       twigs;	            //!< children
	Twig                *parentTwig;            //!< parent twig
	static SbString     *leaders;               //!< variable number of dots for printing 

	Bool                twigWithDialog;         //!< a twig for which createShowDialog was invoked


	enum                set_get {set, get};     //!< flag for set_get operations

	Widget              theParentWidget;        //!< parent widget for the twig
	Widget              ShowDialogShell;        //!< popup shell
	Widget              showWidget;             //!< widget for show menu
	Widget              toggleButton;           //!< visibility toggle button
	Widget              cascadeButton;          //!< menu to make visibility widgets cascade (for traceback)

   	Widget              sub_form;      
  	Widget              sub_show_rowcol;  
  	Widget              formFrame;  
	Widget              ShowRowcol;
	Widget              ShowFormDialog;    

	TwigSeparator       *oiv_node;              //!< OpenInventor separator node  
 static TwigSeparator       *theCurrentIvNode;      //!< marks active node in draw sequence
	std::ostream        *theOstream;            //!< where to route output
  const SoPickedPoint       *pickedPoint;           //!< pick routines store the point here        

        // Protected methods	
        // -----------------
        
        void                showCreateItem    (Widget);

        static void         showCB            (Widget,   XtPointer, XtPointer);

	static void         cascadeCB         (Widget,   XtPointer, XtPointer); //!< callback for cascade button
             
        static void         showCloseCB       (Widget w, XtPointer, XtPointer) //!< callback for close button 
                                               {XtUnmanageChild(XtParent(XtParent(w)));}
                                     
        static void         showRefreshCB     (Widget, XtPointer client_data, XtPointer)  //!< callback for refresh button
                                               {Twig *t = (Twig *)client_data; t->drawTree();
					       };
                                                                                 
	void                printMe();          //!< for tracebacks
	virtual void        printInfo(std::ostream& ostr, const SoNode *node) const;
	virtual void        print(std::ostream& ostr) const;
	virtual void        drawMe(); 
	void                attachChildren();   
	void                recreateIvTree();
	void                destroyTwig();      //!< destroy an entry in twigs list

 	void                getScrollSize(Dimension&, Dimension&); 
        Twig                *findTwigNodeMatch(const SoNode *node);
	Bool                okToRedraw();  //!< basic check of drawability 
};

// Inline defs

inline std::ostream& operator <<(std::ostream &os, const Twig &t) {
  t.print(os);
  return os;
}

// Additional class defs

/*! \class TwigCallbackStruct
 *  \brief Provides a callback structure for Twigs
 *
 * This one is really just a struct for use in X callbacks.
 * We may make this more complicated later so we start with a struct,
 * even if it only has one member.
 */
class TwigCallbackStruct {
public:
	Twig   *theTwig;
	TwigCallbackStruct(Twig *aTwig):  theTwig(aTwig) {};
};	
#endif




















